// Copyright (C) 2005 Dave Griffiths
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <png.h>
#include <stdlib.h>
#include <iostream>
#include <vector>

using namespace std;

void PNGSave(const string &Filename, int &w, int &h, char *data)
{
	FILE *fp=fopen(Filename.c_str(),"wb");
	
	if (!fp) 
	{
		cerr<<"Couldn't open image to write ["<<Filename<<"]"<<endl;
	}
	else
	{
		png_structp png_ptr;
		png_infop info_ptr;
		
		png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   	 	info_ptr = png_create_info_struct(png_ptr);
   		
		png_init_io(png_ptr, fp);

		png_set_IHDR(png_ptr, info_ptr, w, h, 8,
			PNG_COLOR_TYPE_GRAY,PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT,PNG_FILTER_TYPE_DEFAULT);
		
		png_write_info (png_ptr, info_ptr);

		int p=h-1;
		for (int row = 0; row<h; row++)
		{			
			png_write_row (png_ptr, (png_byte*)data+p*w);
			p--;
		}
		
		png_write_end(png_ptr, NULL);
		png_destroy_write_struct(&png_ptr, &info_ptr);	
	}

	fclose(fp);
}

//----------------------------------------------------------------------

string LoadSrc(const string filename)
{
	FILE *srcfile=fopen(filename.c_str(),"r");
	if (!srcfile) 
	{
		cerr<<"file: "<<filename<<" not found"<<endl;
		return NULL;
	}
	
	fseek(srcfile,0,SEEK_END);
	unsigned int size=ftell(srcfile);
	fseek(srcfile,0,SEEK_SET);
	
	char *src=new char[size];
	
	if (size!=fread(src,1,size,srcfile))
	{
		cerr<<"error reading file: "<<filename<<endl;
		fclose(srcfile);
		return NULL;
	}

	fclose(srcfile);
	
	return src;
}



vector<string> Tokenise(const string &in)
{
	unsigned int pos=0;
	bool intoken=false;
	bool incomment=false;
	string token;
	
	vector<string> tokens;	
		
	while (pos<in.size())
	{	
		if ((in[pos]==' ' || in[pos]=='\t' || in[pos]=='\n'))
		{
			if (in[pos]=='\n') incomment=false;
				
			if (intoken) 
			{
				tokens.push_back(token);
				
				token="";
				intoken=false;
			}
		}
		else
		{
			if (in[pos]==';' && !incomment) incomment=true;
			else if (!incomment)
			{
				intoken=true;										
				token.push_back(in[pos]);
			}
		}
		pos++;
	}
	
	return tokens;
}

int main(int argc, char *argv[])
{
	if (argc!=5)
	{
		cerr<<"quagpiler source width height image.png"<<endl;
		return -1;
	}

	int width=(int)atof(argv[2]);
	int height=(int)atof(argv[2]);

	vector<string> in = Tokenise(LoadSrc(argv[1]));

	vector<char> out;
	int count=0;
	for (vector<string>::iterator i=in.begin(); i!=in.end(); i++)
	{
		out.push_back((char)strtod(i->c_str(),NULL));
		count++;
	}

	for (int i=count; i<width*height; i++)
	{
		out.push_back(255);
	}
	
	char *data = &(*out.begin());
	
	PNGSave(argv[4],width,height,data);		
}
